package xyz.leomurca.rickandmorty.data.remote

import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.coroutineScope
import kotlinx.coroutines.withContext
import xyz.leomurca.rickandmorty.data.CharacterRepository
import xyz.leomurca.rickandmorty.data.CharacterResult
import xyz.leomurca.rickandmorty.data.model.Character
import xyz.leomurca.rickandmorty.di.AppDispatchers
import xyz.leomurca.rickandmorty.di.Dispatcher
import xyz.leomurca.rickandmorty.network.NetworkDataSource
import xyz.leomurca.rickandmorty.network.NetworkResult

class RemoteCharacterRepository(
    @Dispatcher(AppDispatchers.IO) private val ioDispatcher: CoroutineDispatcher,
    private val dataSource: NetworkDataSource,
) : CharacterRepository {
    override suspend fun characters(): CharacterResult<List<Character>> {
        return withContext(ioDispatcher) {
            when (val result = dataSource.characters()) {
                is NetworkResult.Success -> {
                    CharacterResult.Success(
                        result.data.results.map {
                            Character(
                                id = it.id,
                                name = it.name,
                                image = it.image
                            )
                        }
                    )
                }

                is NetworkResult.Error -> {
                    CharacterResult.Error(result.errorMessage)
                }
            }
        }
    }
}