package xyz.leomurca.rickandmorty.network.remote

import kotlinx.serialization.json.Json
import xyz.leomurca.rickandmorty.network.NetworkDataSource
import xyz.leomurca.rickandmorty.network.NetworkResult
import xyz.leomurca.rickandmorty.network.model.NetworkCharacter
import xyz.leomurca.rickandmorty.network.model.NetworkCharacterResponse
import xyz.leomurca.rickandmorty.network.model.NetworkError

class RemoteNetworkDataSource(
    private val apiService: ApiService,
    private val json: Json
): NetworkDataSource {
    override suspend fun characters(): NetworkResult<NetworkCharacterResponse> {
        return try {
            val response = apiService.characters()

            if (response.isSuccessful) {
                NetworkResult.Success(response.body() ?: throw Exception("Empty response body"))
            } else {
                val errorBody = response.errorBody()?.string() ?: ""
                val networkError = json.decodeFromString<NetworkError>(errorBody)
                NetworkResult.Error(networkError.message)
            }
        } catch (e: Exception) {
            NetworkResult.Error(e.message.toString())
        }
    }
}