package xyz.leomurca.rickandmorty.ui.home

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch
import xyz.leomurca.rickandmorty.data.CharacterRepository
import xyz.leomurca.rickandmorty.data.CharacterResult
import xyz.leomurca.rickandmorty.data.model.Character
import javax.inject.Inject

@HiltViewModel
class HomeViewModel @Inject constructor(
    private val characterRepository: CharacterRepository
) : ViewModel() {

    private val _uiState = MutableStateFlow<UiState>(UiState.Loading)
    val uiState = _uiState.asStateFlow()

    init {
        viewModelScope.launch {
            _uiState.value = when (val result = characterRepository.characters()) {
                is CharacterResult.Success -> UiState.Loaded.Success(result.data)
                is CharacterResult.Error -> UiState.Loaded.Error(result.message)
            }
        }
    }

    sealed interface UiState {
        data object Loading : UiState

        sealed class Loaded : UiState {
            data class Success(val characters: List<Character>) : Loaded()
            data class Error(val message: String) : Loaded()
        }
    }
}